/********************************************************

	The Aux bus, timer 0 interrupt routines
	Meant to be compiled by Hi-Tech C PICL compiler
	Built for Microchip 16F627/16F628 PIC chips, see as_i2c.h to set which to use

	Copyright 2008 GC
	No direct or derivative commercial use allowed without prior written consent
	All derivative works must include this copyright
	Free for use for private and educational purposes.
	Permission to host this code for distribution in any form on web
	sites other than the authors is not granted.

	See notice at the end of this source code.

	Version 0.1, first published prototype.


*********************************************************/
#include <pic.h>
#include "usart_intr.h"
#include "as_i2c.h"

#if defined( _16F62X)				// Guider, focuser, reticule clone
extern unsigned char rx_buf[];		// Buffer to hold incoming chars from PC, NOT USED
extern unsigned char rx_wr_ptr;		// Next incoming stored here, NOT USED
extern unsigned char rx_rd_ptr;		// Main reads from this ptr, NOT USED
extern unsigned char from_pc_cnt; 		// How many chars Main read in the full message from PC, NOT USED
extern unsigned char send_to_as;		// Main sets this when it has a full message from PC, NOT USED
extern unsigned char from_pc[];		// Where main put the PC data
bank1 extern unsigned char response[];	// Data from AS for focuser and reticule commands
unsigned char send_to_as1 = 0;		// Set by the ISR if Main has set send_to_as, used to sync main and ISR

#elif defined(_12F629)				// Guider only clone
extern unsigned char response[];
#else
#error 1
#endif

extern unsigned char response_wr_ptr;	// We store xmits from AS to where this points
extern unsigned char term_seq_for_0[];	// What we should send if AS does existence poll for us
extern unsigned char term_seq_reticule[];  // What we send if AS sends us a reticule command

unsigned char do_cmd=0;				// AuxClk counter
unsigned char bytecnt1=0;			// Byte count from AS
unsigned char cmdbyte1=0;			// Cmd byte from AS
unsigned char addrbyte1=0;			// Address byte from AS
unsigned long my_timer = 0;			// Count how long since we saw last Aux Clk from AS
unsigned char bitcnt = 0;			// How many bits have we processed
unsigned char hold_sda;				// Holds the current state of the AuxData line read for the pin
unsigned char my_bytecnt = 0;			// How many bytes we need to read from AS for payload data
unsigned char outer_cnt = 0;			// Used to track max # of bytes expected
unsigned char sending_last_bit = 0;		// Set if we are sending the last bit out to the AS
unsigned char my_outcnt = 0;			// How many bytes we will output to AS
unsigned char my_outdata = 0;			// Holds byte we are currently sending to AS
unsigned char my_out_ptr = 0;			// Index into the output to AS array
unsigned char sending_term_seq = 0;		// == 1 if sending a null response to an idle poll when we have nothing from PC to say
unsigned char sending_term_seq_reticule = 0;  // == 1 = sending a '0' at end of reticule cmd from AS
unsigned char sending_ccd_seq = 0;		// Sending guider input info to the AS
unsigned char toggle_me = 0;			// Used for oscopeing debug
unsigned char toggle_int = 0;			
unsigned char toggle_intf = 0;

unsigned char ccd_to_as[5] = {4,0,0,0,0};	// Data to send to AS for guider.
							// Bytes 1 and 2 ffilled in with 16 bit signed in
							// for up/down
							// bytes 3 and 4 same for left/right
int ccd_up_down, ccd_left_right;		// 16 bit signed integer counts
							// maintained in timer ISR code below

#if defined(_16F62X)
int ccd_multiplier = 3;				// timer0 is 1.6ms, for every timer tick
							// we maintain add +/- this value if
							// up/down or left/right input found
#elif defined(_12F629)
int ccd_multiplier = 4;				// ditto for 2ms timer tick
#endif


#if defined( _16F62X)
/**********************************************

	Write a char out the RS232 towards PC

***********************************************/

// Used for debug of focuser and reticule parser for full featured clone
void 
putch(unsigned char byte) 
{
	/* output one byte */
	while(!TXIF)	/* set when register is empty */
		continue;
	TXREG = byte;
}

/**********************************************

	Get a char from the PC.
	Note, this is unsigned int routine,
	we return 0xfff if no character waiting else
	we return the char from the rx buffer

***********************************************/
// NOT USED

#endif
/***********************************************

	The interrupt service routine for:
	1) Aux Clk
	2) Timer0 - ~1.6us interval
	3) AuxClk	on RB0(GP2)/INT

	We build a state machine, reading the first 3 bytes of a message from AS
	AS always start the communications, it polls us for data if we have any to send or
	or asks us if we exist


	do_cmd 	0 = receiving the Byte count byte from the AS
			1 = receiving the Address byte from the AS
			2 = receiving the Command byte from the AS
			>2 = possibly receiving payload to us or sending data to AS


***********************************************/
static void interrupt
isr(void)			// Here be interrupt function - the name is
				// unimportant.
{
#ifdef DEBUG
GPIO5 = toggle_int++;  //Change Me
#endif
						// tuck AUX data away anyway
	if (INTF == 1)			// Aux Clk interrupt comes in on RB0(GP2)/INT pin
						// Falling edge triggered
						// AS clocks data to us on falling AuxClk edge
						// and we gate data oyt to AS on riging edge
	{

	hold_sda = SDA & 0x1;		// Read the state of AuxData

#if defined(_12F629)
#ifdef DEBUG
	GPIO5 = toggle_intf++;	// Change Me
#endif
#endif
		if(sending_last_bit)	// We are sending the last bit out to AS
						// This interrupt should come on the rising edge of last Aux Clk
						// for this sequence
		{
			SDA_LISTEN();	// let go of Aux Data line 
			INTF = 0;		// Ack away the interrupt	// Not sure if need to do this here, we are
											// Changing the criteria for the interrupt
											// from rising edge triggered to falling edge
			INTEDG = 0;		// And now listen for next Aux CLK falling

			// Prime for next sequence from AS

			do_cmd = addrbyte1 = cmdbyte1 = bytecnt1 = sending_last_bit = 0;	// Did I forget sending_last_bit ?
#if defined( _16F62X)
			if(send_to_as1)	// NOT USED< shoudl never be set
			{
				send_to_as = send_to_as1 = from_pc_cnt = my_timer = 0;	// Did I forget from_pc_cnt ?
			}
#endif
			if ((sending_term_seq | sending_ccd_seq) | sending_term_seq_reticule)
			{
				sending_term_seq = sending_ccd_seq = sending_term_seq_reticule = my_timer = 0;
			}
			bitcnt = 8;		// We are all primed, AS may come right back at us
		}
		else if (do_cmd < 8)	// All 909 commands are 8 bytes or less
		{
			hold_sda = SDA & 0x1;	// Grab the state of AuxData (done above also,
							// I found sometimes the data is not stable
							// immediately on ISR entry, so read it again
#if defined(_12F629)
#ifdef DEBUG
			GPIO4 = hold_sda; // Change ME
#endif
#endif
			switch (do_cmd)		// We have at least the first bit in from AS already
								// and now to subsequent ones
			{
			case 0:				// We are working on bytecnt from AS
				bytecnt1 = (bytecnt1 << 1) | hold_sda; break;
			case 1:				// We are working on Addrbyte from AS
				addrbyte1 = (addrbyte1 << 1) | hold_sda;  break;
			case 2: 				// We are working on cmdbyte from AS
				cmdbyte1 = (cmdbyte1 << 1) | hold_sda; break;
			default:				// We have bytecnt, addrbyte and cmd byte already
				if(addrbyte1 == 0x27)	// AS talking to us?
				{
					switch (cmdbyte1)	// Check for AS to us or us to AS
					{
						case 0x1:		// AS to us, NOT USED
						case 0x3:		// AS sent us Focuser command
						case 0x4:		// AS sent us Reticule command
#if defined( _16F62X)
							// Save focuser or reticule command for parsing by main
							response[response_wr_ptr] = (response[response_wr_ptr]) << 1 | hold_sda;
#endif
						case 0x0:		// Possibly us to AS (cmdbyte = 0 says AS poll for us existence
						case 0x5:		// AS poll us for guider input
#if defined( _16F62X)
							// send_to_as1 NOT USED
							if((send_to_as1 | sending_ccd_seq) | (sending_term_seq | sending_term_seq_reticule))	// ==1 = yes, want to send something, this is set below
#elif defined( _12F629)
							if((sending_ccd_seq | sending_term_seq) | sending_term_seq_reticule)	// ==1 = yes, want to send something, this is set below
#else
#error 1
#endif
							{
								SDA = (my_outdata >> (bitcnt - 1));	// Gate out bit
								if(((my_outcnt - 1 ) == 0) & (bitcnt == 1))
								{		// We have sent all but last bit, say are sending last
									INTF = 0;	//Ack away interrupt
									INTEDG = 1;	//And set for trigger on Rising edge AuxClk
									sending_last_bit = 1;	// And say next INT will be a rising edge for the last gated out
								}
							}
							break;
						default:	// Not sure what AS said
my_default:
							break;
					}
				}
				break;
			} // End switch (do_cmd)
			bitcnt--;		// On to next bit

			if (!bitcnt)	// == 0 means we have just finished a bytes worth
			{
				if(do_cmd == 0) outer_cnt = bytecnt1 + 2;	// We just got bytecnt from AS (do_cmd == 0)
											// Byte cnt does not include itself
											// And on a listen does not include the 'float' last byte
											// and on a from AS to us we will stop when we hit this count
				bitcnt = 8; 	// 8 more bits 
				if (do_cmd >= 2)  // We have count and address. Do we also have cmdbyte (do_cmd>=2) ?
				{
					if (addrbyte1 == 0x27)	// AS talking to us?
					{
						if(do_cmd == 2)	// We just got last bit of cmdbyte on falling edge of AuxClk
						{
							switch(cmdbyte1)	// Now what was the cmd AS sent us?
							{
								case 0x0:		// AS polled us, say we are here
										my_outcnt = term_seq_for_0[0] + 1;	// out count + 1
										my_outdata = term_seq_for_0[0];	// First byte sent is bytecnt
										sending_term_seq = 1;	// Tell ourselves to send guide input info term seq to AS
gohere:
										my_out_ptr = 1;			// index to out to AS array
										SDA = hold_sda;			// Get the current Aux Data line state
														// and put it back out on the line when we take control of it
										SDA_TALK();				// Grab the Aux Data line
										INTEDG = 1;	//And set for trigger on Rising edge AuxClk
										INTF = 0;	//Ack away interrupt

										sending_last_bit = 0;		// When = 1 last bit has been sent
										outer_cnt = outer_cnt + my_outdata;	// cnt,addr,cmd,dummy when listening
																// We start talking in the dummy period
																// And always send a min of 2 bytes
																// First byte in the 'dummy' period
										if(toggle_me)
										{
											toggle_me = 0;
											goto gohere1;
										}

									break;
								case 0x3: // Reticule command from AS
								case 0x4: // Focuser command from AS
									my_bytecnt = bytecnt1 - 2;	// bytecnt includes addr and cmdbyte
#if defined( _16F62X)
									// Save data for parsing by main
									response[response_wr_ptr++] = bytecnt1;
									response_wr_ptr &= RESPONSE_BUF_MASK;	// Point to next place to store data
									response[response_wr_ptr++] = addrbyte1;
									response_wr_ptr &= RESPONSE_BUF_MASK;
									response[response_wr_ptr++] = cmdbyte1;
									response_wr_ptr &= RESPONSE_BUF_MASK;
									response[response_wr_ptr] = 0;	// Start with a 0
#endif
									break;
								case 0x5: // AS poll for CCD guider input
#if defined( _16F62X)
									// send_to_as NOT USED
									if(send_to_as)		// Did Main tell us we have something to send?
									{
										send_to_as1 = 1;	// Tell ourselves (sync external code with ISR)
										my_outcnt = from_pc_cnt + 1;	// How many payload to send
										my_outdata = from_pc_cnt;	// First byte sent is bytecnt
										my_out_ptr = 0;			// index to out to AS array
										SDA = hold_sda;			// Get the current Aux Data line state
															// and put it back out on the line when we take control of it
										SDA_TALK();				// Grab the Aux Data line
										INTF = 0;	//Ack away interrupt
										INTEDG = 1;	//And set for trigger on Rising edge AuxClk
// Not sure if we should gate out our bit yet			SDA = (my_outdata >> 7 ) & 0x1;
// We MUST gate out here on the 4MHZ part

										sending_last_bit = 0;		// When = 1 last bit has been sent
										outer_cnt = outer_cnt + from_pc_cnt;
									}
									else	// Nothing from main, send out default terminating sequence to AS poll
#endif	// defined(_16F62X)
									{
										sending_ccd_seq = 1;	// Tell ourselves to send guide input info term seq to AS
										my_outcnt = ccd_to_as[0] + 1;	// out count + 1
										my_outdata = ccd_to_as[0];	// First byte sent is bytecnt
										my_out_ptr = 1;			// index to out to AS array
										ccd_to_as[1] = (unsigned char)(ccd_up_down >> 8);
										ccd_to_as[2] = (unsigned char)(ccd_up_down & 0xff);
										ccd_to_as[3] = (unsigned char)(ccd_left_right >> 8);
										ccd_to_as[4] = (unsigned char)(ccd_left_right & 0xff);
										ccd_up_down = ccd_left_right = 0;

										SDA = hold_sda;			// Get the current Aux Data line state
										SDA_TALK();			// Grab the Aux Data line
										// Jam our first bit out
										// Need to do this for slow 4Mhz clock
										// There may be slight AuxData line
										// contention, current limit resistors
										// in the line on both clone
										// and AS handle it
										SDA = (my_outdata >> 7 ) & 0x1;
														// and put it back out on the line when we take control of it

										INTF = 0;			//Ack away AuxClk interrupt
										INTEDG = 1;			//And set for trigger on Rising edge AuxClk

										sending_last_bit = 0;		// When = 1 last bit has been sent
										outer_cnt = outer_cnt + my_outdata;	// cnt,addr,cmd,dummy when listening
																// We start talking in the dummy period
																// And always send a min of 2 bytes
																// First byte in the 'dummy' period
										if(toggle_me)
										{
											toggle_me = 0;
											goto gohere1;
										}
									}
									break;
								default:
my_default1:								// Not sure what AS said
									break;
							}
						} 
						else		// May be receiving or sending here
						{
							if(my_bytecnt)	// We are receiving focuser or
										// reticule command
							{
								my_bytecnt--;
#if defined( _16F62X)
								// Point to next place to put data
								response_wr_ptr++; response_wr_ptr &= RESPONSE_BUF_MASK;	// Point to next place to store data
#endif
								if(!my_bytecnt)
								{
									if(cmdbyte1 == 3)	// This is a kludgy at putting out a '0' at the end of every AS write to reticule
												// but it works
									{
										sending_term_seq_reticule = 1;
										my_outcnt = term_seq_reticule[0] + 1;	// out count + 1
										my_outdata = term_seq_reticule[0];	// First byte sent is bytecnt
										do_cmd = 3;
										outer_cnt = 0;
										toggle_me = 1;
										goto gohere;
									}
								}
							}
#if defined( _16F62X)				// Next else if NOT USED
							else if (send_to_as1) // Are we sending something from the PC?
							{
								my_outdata = from_pc[my_out_ptr++];	// get next byte to send
								my_outcnt--;
							}
#endif
							else if (sending_ccd_seq)	// Are we sending the idle response to AS when it polls?
							{
								my_outdata = ccd_to_as[my_out_ptr++];
								my_outcnt--;
							}
							else if (sending_term_seq)	// Are we sending the response to AS when it polls
												// us for existence?
							{
								my_outdata = term_seq_for_0[my_out_ptr++];
								my_outcnt--;
							}
							else if (sending_term_seq_reticule)	// Are we sending the response to say we heard
													// the reticule command
							{
								my_outdata = term_seq_reticule[my_out_ptr++];
								my_outcnt--;
							}
						}
					}
				}
				do_cmd++;	// Move on to next byte from AS
				if (!sending_last_bit)	// If this is an xmit to AS and we are in process of sending last bit
								// then we wait for rising edge of last AuxClk for this sequence
								// Once we get it, we will clean up on entry to ISR
				{
					if( (do_cmd >= outer_cnt) )	// All done, clean up
					{
						do_cmd = addrbyte1 = cmdbyte1 = bytecnt1 = 0;
						SDA_LISTEN(); INTF = 0; INTEDG = 0; // Just in case!
						bitcnt = 8;
					}
				}
gohere1:
				if (do_cmd > 31) do_cmd = 31; // Just in CASE!!!!
			} // Endif my_timer
			my_timer = 0;
		}
		else if((my_timer < 4000))
		{
//			AN idle for up to 4000 Timer0 interrupts for comms to the AS.
			if (my_timer > 2)	// We run a 1.6ms timer. If no INT in 3.2ms then
						// we were not active with send or receive
						// And this is first bit of new Xfer from AS
			{
				bitcnt = 7;				// We just got first bit so 7 more of this sequence for bytecnt from AS
				my_timer = 0;
				do_cmd = addrbyte1 = cmdbyte1 = 0;	// Make sure we are primed correctly
				INTEDG = 0;				// And listen for Aux CLK falling
				SDA_LISTEN(); 			// Listen on Aux Data pin
				hold_sda = SDA & 0x1;
				bytecnt1 = hold_sda;		// Get the first bit AS just sent us, this wil become the MSB of the bytecnt
				sending_last_bit = 0;		// Never know when may lose comms or AS saying nothing
#if defined( _16F62X)
				if(send_to_as1) { send_to_as1 = send_to_as = from_pc_cnt = 0; }	// Just in case!!! send_to_as = Did I forget from_pc_cnt? was not there before
#endif
				if((sending_ccd_seq | sending_term_seq) | sending_term_seq_reticule) { sending_ccd_seq = sending_term_seq = sending_term_seq_reticule = 0; }	// Just in case!!!
			}
		} 
		else 
		{
			// This hits if we had more than 4000 Timer0 ticks, this forces us to wait 4000 more
			// before trying again
			if(my_timer < (unsigned long)-4000)
				my_timer = -4000L;	// Give AS time to come up, ISR will apply pullup
			goto done1;
		}
done1:
		INTF = 0;
	}

#if defined( _16F62X)
/***************************************************

	Receive serial interrupt, NOT USED

***************************************************/
	if (RCIF == 1)
	{
		rx_buf[rx_wr_ptr++] = RCREG;		// HW clears the RCIF bit when we read RCREG
		rx_wr_ptr &= 0x1f;
	}
#endif
/**************************************************

Timer0 interrupt

**************************************************/
	if (T0IF == 1)
	{

		if (my_timer > 8)	// We run a 1.6ms timer. If no INT in 3.2ms then
						// we were not active with send or receive
						// And this is first bit of new Xfer from AS
		{
			bitcnt = 8;				// We just got first bit so 7 more of this sequence for bytecnt from AS
			do_cmd = addrbyte1 = cmdbyte1 = 0;	// Make sure we are primed correctly
			INTEDG = 0;				// And listen for Aux CLK falling
			SDA_LISTEN(); 			// Listen on Aux Data pin
			sending_last_bit = 0;		// Never know when may lose comms or AS saying nothing
#if defined( _16F62X)
			// send_to_as1 NOT USED
			if(send_to_as1) { send_to_as1 = send_to_as = from_pc_cnt = 0; }	// Just in case!!! send_to_as = Did I forget from_pc_cnt? was not there before
#endif
			if(sending_ccd_seq | sending_term_seq) { sending_ccd_seq = sending_term_seq = 0; }	// Just in case!!!
		}

// We check the input pins for = 0, if = 0 then pin asserted
// and we count
		if(!CCD_UP) ccd_up_down += ccd_multiplier;
		if(!CCD_DOWN) ccd_up_down -= ccd_multiplier;
		if(!CCD_LEFT) ccd_left_right -= ccd_multiplier;
		if(!CCD_RIGHT) ccd_left_right += ccd_multiplier;
		my_timer++; T0IF = 0;
	}
} // interrupt routine done

/*
All downloads and use of the code are subject to the following terms: Software and
source code Copyright (C) 2008 GC. This software is copyright under the following
conditions: Permission to use, copy, and modify this software and its documentation is
hereby granted to all academic and not-for-profit institutions and individuals without fee, provided that
the above copyright notice and this permission notice appear in all copies of the software
and related documentation. Permission to distribute the software verbatim is NOT explicitly granted.
Significantly modified or extended
versions thereof on a not-for-profit basis is explicitly granted, under the above conditions.
However, the right to use this software by companies or other for profit organizations, or
in conjunction with for profit activities, and the right to distribute the software or
modified or extended versions thereof for profit are NOT granted except by prior
arrangement and written consent of the copyright holder. For these purposes, downloads
of the source code constitute "use" and downloads of this source code by for profit
organizations and/or distribution to for profit institutions in explicitly prohibited without
the prior consent of the copyright holder. Use of this source code constitutes an
agreement not to criticize, in any way, the code-writing style of the author, including any
statements regarding the extent of documentation and comments present. The software is
provided "AS-IS" and without warranty of ank kind, express, implied or otherwise,
including without limitation, any warranty of merchantability or fitness for a particular
purpose. In no event shall the author be liable for any special,
incudental, indirect or consequential damages of any kind, or any damages whatsoever
resulting from loss of use, data or profits, whether or not advised of the possibility of
damage, and on any theory of liability, arising out of or in connection with the use or
performance of this software.
*/
