/************************************************************************

	Main routine for Autostar (Trademark of Meade Corp) 909 cable functionality
	Meant to be compiled by Hi-Tech C PICL compiler
	Built for Microchip 16F627/16F628 or 12F629 PIC chips

	Copyright 2008 GC
	No direct or derivative commercial use allowed without prior written consent
	All derivative works must include this copyright
	Free for use for private and educational purposes.
	See notice at the end of this source code.
	Permission to host this code for distribution in any form on web
	sites other than the authors is not granted.

	Version 0.2: Focuser code added matching schematic on web site

	Version 0.1, first published prototype for Meade 909 guider clone.


*************************************************************************/


#include <stdio.h>
#include <pic.h>
#include "usart_intr.h"
#include "as_i2c.h"

#define OSC_FREQ	20000000L		// Main clock speed = 20Mhz, NOT USED
extern unsigned long my_timer;		// Used to count Timer0 2ms ticks

// State machine values for parsing Focuser and Reticule, not used for Guider Only
#define GET_FOCUSER_DIR		5
#define GET_FOCUSER_BYTE1	6
#define GET_FOCUSER_BYTE2	7
#define GET_FOCUSER_BYTE3	8
#define GET_FOCUSER_BYTE4	9
#define GET_RETICULE_DC		10
#define GET_RETICULE_IH		11
#define GET_RETICULE_IL		12
#define GET_RETICULE_RH		13
#define GET_RETICULE_RL		14

// None of the below are used for anything in the guider only clone but they are filled in

unsigned char focuser_dir = 0;
unsigned char focuser_speed = 0;
unsigned char reticule_dc = 0;		// Duty cycle
unsigned char reticule_ih = 0;		// Intensity high byte
unsigned char  reticule_il = 0;		// Intensity low byte
unsigned char reticule_rh = 0;		// Rate high byte
unsigned char reticule_rl = 0;		// Rate low byte

#if defined( _16F62X)

bank1 unsigned char from_pc[32];	// Use second PIC ram bank to hold data from PC
						// For future use

bank1 unsigned char response[RESPONSE_BUF_SIZE];	// Use second PIC ram bank to hold
						// raw data from Focuser and Reticule commands
unsigned char my_count;

unsigned char rx_wr_ptr = 0;		// Receive from PC write index, future use
unsigned char rx_rd_ptr = 0;		// Take out of PC buffer read index, future use

unsigned char from_pc_cnt = 0;	// How many bytes we got from PC and need to be sent to AS, future use
unsigned char send_to_as = 0;		// Used to tell ISR we have something to send to AS, future use
unsigned char must_send = 0;		// COunt of number of bytes for a 'download mode' command, NOT used

//unsigned char ccd_port = 0;
//unsigned char old_ccd_port = 0;
unsigned char which_bits = 0;
unsigned char as_bytecnt, as_addrbyte, as_cmdbyte;
unsigned char state = 0;
						// responses from AS
unsigned char rx_buf[15];		// Holds data received from PC in the ISR, FUTURE USE

#elif defined(_12F629)			// 8 pin PIC for guider only version
unsigned char response[RESPONSE_BUF_SIZE];	// This is a bit bucket for the guider only 909 clone
#ifdef DEBUG
unsigned char toggle_main = 0; // debug of 8pin'er, main counter
#endif
#endif

unsigned char response_rd_ptr = 0;	// Receive from AS rd index, not used in guider only
unsigned char response_wr_ptr = 0;	// Receive from AS wr index, not used in guider only

unsigned char term_seq_for_0[2] = {0x1,0x2};	// What to send to AS when AS polls for existence
								// bytecount to send, data to send

unsigned char term_seq_reticule[2] = {0x0, 0x0}; // What to send as response to Reticule command


// Set up the PIC configuration

#if defined( _16F62X)
__CONFIG(WDTDIS & LVPEN & PWRTEN & MCLRDIS & BORDIS & DATUNPROT & UNPROTECT & EC);
// Change the EC above to INTIO for 4MHZ internal clock
#elif defined( _12F629)
__CONFIG(WDTDIS & PWRTEN & MCLRDIS & BORDIS & UNPROTECT & INTIO);
// COnfigured for 4Mhz interal oscillator
#else
#error 1
#endif

void main(void)
{
	unsigned int input;		// Holds byte of data from PC, reason for 'int' explained below

// Set up the PIC chip

	INTCON=0;				// Disable all interrupts.
	OPTION = 0b10000100;		//  pull ups disabled,
						// prescale timer0 by 32 for ~ 1.6ms/interrupt @20MHZ (/32)
						// Set to 0bxxxxx010 for 4MHZ = ~2ms/interrupt @ 4MHZ (/8) (see below for 12F629 code)
						// Also see usart_intr.h for baud rate clock if using 16f62X
	T0CS = 0;				// select internal clock for Timer0

	T0IE = 1;				// Enable interrupt on TMR0 overflow
	INTEDG = 0; 			// Set falling edge triggered interrupt on RB0(GP2)/INT
	CMCON = 0x7; 			// Comparators off


#if defined( _16F62X)
	TRISA = (unsigned char)0xff;	// Start with port A and B set to inputs
	TRISB = (unsigned char)0xff;
	PORTA = (unsigned char)0xff;
	PORTB = (unsigned char)0xff;
	RB5 = 0; 	// Disable focuser power
	TRISB5 = 0;	// Focuser Enable pin to output mode
	TRISB6 = 0;	// Focuser == 1 = clockwise, == 0 = counter clockwise
	TRISB7 = 0; // Focuser == 1 = counterclockwise, == 0 == clockwise
	RA6 = 1;	// Set to High volt to start with, nothing gated because power turned off
	TRISA6 = 0;

#elif defined( _12F629)
	OPTION = 0b00000010;		// GPIO pull ups enabled, timer0 for a 2ms clock
	TRISIO = (unsigned char)0xff;	// All pins are inputs

#ifdef DEBUG

	TRISA0 = 0; // use for oscoping outputs
	TRISA4 = 0; 
	TRISA5 = 0; 
#endif
	GPIO = (unsigned char)0xff;		// Set all pins to input
	WPU = (unsigned char)0x37;		// Second level weak pull up enable
	OSCCAL = _READ_OSCCAL_DATA();	// Use factory calibration for the internal 4Mhz OSC
						// DO NOT wipe out your OSCCAL from last place in memory when you prog your device!!!
#else
#error 1 // Not defined chip!
#endif

	SCL_LISTEN();			// Set for listening on AuxClk and AuxData
	SDA_LISTEN();

#if defined(_16F62X)
// The folliwng serial output used for debugging the focuser and reticule parser
	init_comms();			// set up the USART - settings defined in usart.h
	RCIE = 1;      			// USART Receive Interrupt Enable
#endif
	INTE = 0; 				// Disable external RB0(GP2)/INT pin

	//Set global Interrupt Enable bits
	GIE = 1;     			// global interrput enable
	PEIE = 1;    			// Peripheral Interrupt Enable bit...1 = Enables all unmasked peripheral interrupts

// Now do some real work
	while (my_timer < 500) CCD_UP = 1; ; // 500 * 1.6ms(2ms)Timer0 = ~0.8/1.0 seconds, give AS a chance to start
	my_timer = 0;
	INTF = 0;		// Clear any pending INT from AS Aux CLk
	INTE = 1;		// Enable external RB0(GP2)/INT pin
	INTF = 0;		// CLear it again!

	while(1)		// In the guider only version, this is all there is in main
	{ 
#if defined(_12F629)
#ifdef DEBUG
GPIO0 = toggle_main++; // For oscope debugging
#endif
#endif
#if defined( _16F62X)
		if(!send_to_as)		// Will be reset by ISR when bytes have been sent
						// Currently NOT USED or ever set
		{
			if(response_rd_ptr != response_wr_ptr) // Did we get a focuser or reticule command from AS
			{
				switch (state)
				{
					case 0x0: as_bytecnt = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK; state++; break;
					case 0x1: as_addrbyte = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK; state++; break;
					case 0x2: as_cmdbyte = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
							if(as_addrbyte == 0x27) // The 909
							{
								switch (as_cmdbyte)
								{
									case 4:		//Focuser command from AS
										state = GET_FOCUSER_DIR; // First byte after CMD is DIR
										break;
									case 3:
										state = GET_RETICULE_DC; // Next thing following is Duty Cycle
										break;
									default:	// Should NOT get here, info filled in by our own ISR
										break;
								}
							}
						break;
					case GET_FOCUSER_DIR:
						state = GET_FOCUSER_BYTE1; // First byte after CMD is DIR
						focuser_dir = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_FOCUSER_BYTE1:		//Focuser command from AS
						state = GET_FOCUSER_BYTE2; // First byte after CMD is DIR
						response_rd_ptr++; response_rd_ptr &= RESPONSE_BUF_MASK;
						// This byte is always zero so nothing to do with it
						break;
					case GET_FOCUSER_BYTE2:		//Focuser command from AS
						state = GET_FOCUSER_BYTE3; // Second byte after CMD is speed (1-4)
						focuser_speed = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_FOCUSER_BYTE3:		//Focuser command from AS
						state = GET_FOCUSER_BYTE4; 
						response_rd_ptr++; response_rd_ptr &= RESPONSE_BUF_MASK;
						// This byte is always zero so nothing to do with it
						break;
					case GET_FOCUSER_BYTE4:		//Focuser command from AS
						// This byte is XOR of BYTE1 so ignore it
						state = 0; // Got all bytes
						response_rd_ptr++; response_rd_ptr &= RESPONSE_BUF_MASK;
					// DO REAL WORK HERE in future rev of code for full featured clone
						putch('F'); // For now, just send what we got to PC
						putch('<');
						putch(focuser_dir);
						putch(focuser_speed);
						putch('>');
						if(focuser_speed == 0)
						{
							RB5 = 0;	// Disable focuser power
						}
						else
						{	
							if (focuser_dir == 1) { RB6 = 1; RB7 = 0; }	// clockwise
							else { RB6 = 0; RB7 = 1; }					// counter clockwise
							if(focuser_speed <= 2) RA6 = 0;				// low volts
							else RA6 = 1;								// High volts
							RB5 = 1;									// Enable power
						}

						break;
					case GET_RETICULE_DC:
						state = GET_RETICULE_IH;	// Next byte is Intensity high byte
						reticule_dc = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_RETICULE_IH:
						state = GET_RETICULE_IL;	// Next byte is Intensity low byte
						reticule_ih = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_RETICULE_IL:
						state = GET_RETICULE_RH;	// Next byte is rate high byte
						reticule_il = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_RETICULE_RH:
						state = GET_RETICULE_RL;	// Next byte is rate high byte
						reticule_rh = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
					case GET_RETICULE_RL:
						state = 0;	// Got all bytes of this command
						reticule_rl = response[response_rd_ptr++]; response_rd_ptr &= RESPONSE_BUF_MASK;
					// DO REAL WORK HERE in future rev of code for full featured clone
						putch('R');		// For now, just send what we got to PC
						putch('<');
						putch(reticule_dc);
						putch(reticule_ih);
						putch(reticule_il);
						putch(reticule_rh);
						putch(reticule_rl);
						putch('>');
						break;
					default:	// // Should NOT get here, info filled in by our own ISR
						response_rd_ptr++; response_rd_ptr &= RESPONSE_BUF_MASK;
						break;
				}
			}
		}
#endif
	} /* End While */
}


// Thats all folks!!!

/*
All downloads and use of the code are subject to the following terms: Software and
source code Copyright (C) 2008 GC. This software is copyright under the following
conditions: Permission to use, copy, and modify this software and its documentation is
hereby granted to all academic and not-for-profit institutions and individuals without fee, provided that
the above copyright notice and this permission notice appear in all copies of the software
and related documentation. Permission to distribute the software verbatim is NOT explicitly granted.
Significantly modified or extended
versions thereof on a not-for-profit basis is explicitly granted, under the above conditions.
However, the right to use this software by companies or other for profit organizations, or
in conjunction with for profit activities, and the right to distribute the software or
modified or extended versions thereof for profit are NOT granted except by prior
arrangement and written consent of the copyright holder. For these purposes, downloads
of the source code constitute "use" and downloads of this source code by for profit
organizations and/or distribution to for profit institutions in explicitly prohibited without
the prior consent of the copyright holder. Use of this source code constitutes an
agreement not to criticize, in any way, the code-writing style of the author, including any
statements regarding the extent of documentation and comments present. The software is
provided "AS-IS" and without warranty of ank kind, express, implied or otherwise,
including without limitation, any warranty of merchantability or fitness for a particular
purpose. In no event shall the author be liable for any special,
incudental, indirect or consequential damages of any kind, or any damages whatsoever
resulting from loss of use, data or profits, whether or not advised of the possibility of
damage, and on any theory of liability, arising out of or in connection with the use or
performance of this software.
*/